<?php

/**
 * Copyright (c) 2014 - 2016. ООО "БАРС - 46"
 */
class CPHPCacheRedis implements ICacheBackend
{
    /**
     * @var Redis $obRedis
     */
    private static $obRedis;

    /**
     * @var array массив с хешами base_dir
     */
    private static $basedir_version = array();

    /**
     * @var string соль для ключей кеша
     */
    var $sid = "";

    //cache stats
    var $written = false;
    var $read = false;

    /**
     * CPHPCacheRedis constructor.
     */
    function __construct()
    {
        $this->CPHPCacheRedis();
    }

    /**
     * Инициализация соединения с redis
     */
    function CPHPCacheRedis()
    {
        if (!is_object(self::$obRedis)) {
            $redisIP = (defined('BX_REDIS_IP') ? BX_REDIS_IP : '127.0.0.1');
            $redisPort = (defined('BX_REDIS_PORT') ? BX_REDIS_PORT : '6379');
            self::$obRedis = new Redis();
            self::$obRedis->connect($redisIP, $redisPort);
        }

        if (!defined("BX_REDISCACHE_CONNECTED")) {
            if (self::$obRedis->ping() == '+PONG') {
                define("BX_REDISCACHE_CONNECTED", true);
                register_shutdown_function(array("CPHPCacheRedis", "close"));
            }
        }

        if (defined("BX_CACHE_SID")) {
            $this->sid = BX_CACHE_SID;
        } else {
            $this->sid = "BX";
        }
    }

    /**
     * Закрытие соединения с redis
     */
    function close()
    {
        if (defined("BX_REDISCACHE_CONNECTED") && is_object(self::$obRedis)) {
            self::$obRedis->close();
        }
    }

    /**
     * @return bool флаг наличия соединения с redis
     */
    function IsAvailable()
    {
        return defined("BX_REDISCACHE_CONNECTED");
    }

    /**
     * Очистка кеша
     *
     * @param $basedir
     * @param bool $initdir
     * @param bool $filename
     *
     * @return bool
     */
    function clean($basedir, $initdir = false, $filename = false)
    {
        if (is_object(self::$obRedis)) {
            if (strlen($filename)) {
                if (!isset(self::$basedir_version[$basedir])) {
                    self::$basedir_version[$basedir] = self::$obRedis->get($this->sid . $basedir);
                }

                if (self::$basedir_version[$basedir] === false || self::$basedir_version[$basedir] === '') {
                    return true;
                }

                if ($initdir !== false) {
                    $initdir_version = self::$obRedis->get(self::$basedir_version[$basedir] . "|" . $initdir);
                    if ($initdir_version === false || $initdir_version === '') {
                        return true;
                    }
                } else {
                    $initdir_version = "";
                }

                self::$obRedis->del(self::$basedir_version[$basedir] . "|" . $initdir_version . "|" . $filename);
            } else {
                if (strlen($initdir)) {
                    if (!isset(self::$basedir_version[$basedir])) {
                        self::$basedir_version[$basedir] = self::$obRedis->get($this->sid . $basedir);
                    }

                    if (self::$basedir_version[$basedir] === false || self::$basedir_version[$basedir] === '') {
                        return true;
                    }

                    $initdir_version = self::$obRedis->get(self::$basedir_version[$basedir] . "|" . $initdir);
                    if ($initdir_version === false || $initdir_version === '') {
                        return true;
                    }

                    $keys = self::$obRedis->keys(self::$basedir_version[$basedir] . "|" . $initdir_version . "|*");
                    self::$obRedis->del($keys);
                    self::$obRedis->del(self::$basedir_version[$basedir] . "|" . $initdir);
                } else {
                    if (!isset(self::$basedir_version[$basedir])) {
                        self::$basedir_version[$basedir] = self::$obRedis->get($this->sid . $basedir);
                    }

                    if (self::$basedir_version[$basedir] === false || self::$basedir_version[$basedir] === '') {
                        return true;
                    }

                    $keys = self::$obRedis->keys(self::$basedir_version[$basedir] . "|*");
                    self::$obRedis->del($keys);
                    self::$obRedis->del($this->sid . $basedir);
                    if (isset(self::$basedir_version[$basedir])) {
                        unset(self::$basedir_version[$basedir]);
                    }
                }
            }

            return true;
        }

        return false;
    }

    /**
     * Чтение пары ключ/значение из кеша
     *
     * @param $arAllVars
     * @param $basedir
     * @param $initdir
     * @param $filename
     * @param $TTL
     *
     * @return bool
     */
    function read(&$arAllVars, $basedir, $initdir, $filename, $TTL)
    {
        if (!isset(self::$basedir_version[$basedir])) {
            self::$basedir_version[$basedir] = self::$obRedis->get($this->sid . $basedir);
        }

        if (self::$basedir_version[$basedir] === false || self::$basedir_version[$basedir] === '') {
            return false;
        }

        if ($initdir !== false) {
            $initdir_version = self::$obRedis->get(self::$basedir_version[$basedir] . "|" . $initdir);
            if ($initdir_version === false || $initdir_version === '') {
                return false;
            }
        } else {
            $initdir_version = "";
        }

        $arAllVars = json_decode(self::$obRedis->get(self::$basedir_version[$basedir] . "|" . $initdir_version . "|" . $filename),
            true);

        if ($arAllVars === null || $arAllVars === '') {
            return false;
        }

        return true;
    }

    /**
     * Запись пары ключ/значение в кеш
     *
     * @param $arAllVars
     * @param $basedir
     * @param $initdir
     * @param $filename
     * @param $TTL
     */
    function write($arAllVars, $basedir, $initdir, $filename, $TTL)
    {
        if (!isset(self::$basedir_version[$basedir])) {
            self::$basedir_version[$basedir] = self::$obRedis->get($this->sid . $basedir);
        }

        if (self::$basedir_version[$basedir] === false || self::$basedir_version[$basedir] === '') {
            self::$basedir_version[$basedir] = $this->sid . md5(mt_rand());
            self::$obRedis->set($this->sid . $basedir, self::$basedir_version[$basedir]);
        }

        if ($initdir !== false) {
            $initdir_version = self::$obRedis->get(self::$basedir_version[$basedir] . "|" . $initdir);
            if ($initdir_version === false || $initdir_version === '') {
                $initdir_version = $this->sid . md5(mt_rand());
                self::$obRedis->set(self::$basedir_version[$basedir] . "|" . $initdir, $initdir_version);
            }
        } else {
            $initdir_version = "";
        }

        self::$obRedis->set(self::$basedir_version[$basedir] . "|" . $initdir_version . "|" . $filename,
            json_encode($arAllVars), intval($TTL));
    }

    /**
     * Унаследовано от интерфейса ICacheBackend всегда возвращает true
     *
     * @param $path
     *
     * @return bool
     */
    function IsCacheExpired($path)
    {
        return false;
    }
}
